/*
	Kotoran's Shadow, an Amstrad game for the CPC RetroDev Contest 2014
    Copyright (C) 2014  Kronoscide

	This file is part of Kotoran's Shadow.

    Kotoran's Shadow is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <cpcrslib.h>
#include <stdio.h>
#include <stdlib.h>
#include "globals.h"
#include "sprites.h"
#include "maps.h"
#include "tiles.h"
#include "intro.h"
#include "menu.h"
#include "game.h"
#include "pause.h"
#include "end.h"
#include "fondos.h"  

#include "cpcwyzlib.h"
#include "datos.h"  

void changePalette(unsigned char k){
	unsigned char i;
	const unsigned char col[6][16] = {
	//    0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15
		{ 1, 24, 20,  6, 26,  0,  2,  8, 10, 12, 14, 16, 18, 22, 13,  9},
		{ 0, 24, 12, 13,  9, 26, 10,  6, 10, 12, 14, 16, 18, 22, 13,  9},
		{ 1, 12,  0, 13, 24, 26, 10,  6, 10,  0, 14, 16, 18, 22, 13, 24},
		{ 0, 12,  6, 13, 26, 26, 10,  6, 10,  6, 14, 16, 18, 22, 13, 26},
		{ 0, 12, 13, 26, 24, 26, 10,  6, 10, 13, 14, 16, 18, 22, 26, 24},
		{ 0,  6, 12, 13,  9, 26, 10,  6, 10, 12, 14, 16, 18, 22, 13,  9}
	};

	for (i=0; i<16; i++) {
		cpc_SetInk(i, col[k][i]);
	}

}

// Inicializador General
void Init () {
	cpc_SetModo(0);
	cpc_SetBorder(0);
	changePalette(0);
	
	cpc_AssignKey(KEY_UP, 0x4708);
	cpc_AssignKey(KEY_RIGHT, 0x4720);
	cpc_AssignKey(KEY_DOWN, 0x4710);  
	cpc_AssignKey(KEY_LEFT, 0x4820);
	cpc_AssignKey(KEY_ESCAPE, 0x4804);
	cpc_AssignKey(KEY_Z, 0x4880);
	cpc_AssignKey(KEY_X, 0x4780);
	cpc_AssignKey(KEY_SPACE, 0x4580);
	cpc_AssignKey(KEY_ENTER, 0x4204);
}

void main () {

	unsigned char aux = 0;
	
	Init();
	
	State = STATE_INTRO;
	Initialized = 0;
	
	cpc_WyzInitPlayer(SOUND_TABLE_0, RULE_TABLE_0, EFFECT_TABLE, SONG_TABLE_0);
	cpc_WyzLoadSong(0);	
	cpc_WyzSetPlayerOn();
	
	while (1) {
	
		if(EfecDisp){
			cpc_WyzStartEffect(2, 2);
			EfecDisp = 0;
		}
		
		if(EfecCogerVida){
			cpc_WyzStartEffect(2, 0);
			EfecCogerVida = 0;
		}
		 
		if(EfecDano){
			cpc_WyzStartEffect(2, 1);
			EfecDano = 0;
		}
		
		if(EfecBoton){
			cpc_WyzStartEffect(2, 3);
			EfecBoton = 0;
		}
		
		if(EfecDanoEnemy){
			cpc_WyzStartEffect(2, 4);
			EfecDanoEnemy = 0;
		}
	
		if(EfecDispEnemy){
			cpc_WyzStartEffect(1, 5);
			EfecDispEnemy = 0;
		}

		if (State == STATE_INTRO) {
	
			if (!Initialized){
				changePalette(5);
				InitIntro();
				cpc_UnExo(Kronoscide,0xC000);
			}
			UpdateIntro();
			
		} else if (State == STATE_MENU) {
			
			if (!Initialized){
				changePalette(1);			
				InitMenu();
				cpc_UnExo(Menu,0xC000);			
			}

			if(Music){
			
				cpc_WyzSetPlayerOff();
				cpc_WyzInitPlayer(SOUND_TABLE_0, RULE_TABLE_0, EFFECT_TABLE, SONG_TABLE_0);
				cpc_WyzLoadSong(0);	
				cpc_WyzSetPlayerOn();
				cpc_WyzStartEffect(0, 4);
				Music = 0;
				
			}
			
			UpdateMenu();
			
		} else if (State == STATE_PAUSE) {
		
			if (!Initialized){
				changePalette(2);
				InitPause();
				cpc_UnExo(Pause,0xC000);
			}
			UpdatePause();
			
		}  else if (State == STATE_END) {
			aux = 3 + Win;
			changePalette(aux);
			if (!Initialized){				
				InitEnd();				
				if(Win){
					cpc_UnExo(End,0xC000);
				}
				else{
					cpc_UnExo(Death,0xC000);
				}
			}
			
			if(Music){
				
				if(Win){
					cpc_WyzSetPlayerOff();
					cpc_WyzInitPlayer(SOUND_TABLE_4, RULE_TABLE_4, EFFECT_TABLE, SONG_TABLE_4);
					cpc_WyzLoadSong(0);	
					cpc_WyzSetPlayerOn();
					Music = 0;
				}
				else{
					cpc_WyzSetPlayerOff();
					cpc_WyzInitPlayer(SOUND_TABLE_5, RULE_TABLE_5, EFFECT_TABLE, SONG_TABLE_5);
					cpc_WyzLoadSong(0);	
					cpc_WyzSetPlayerOn();
					Music = 0;
				}
				
			} 
			
			UpdateEnd();
			
		} else if (State == STATE_GAME) {
		
			if (!Initialized){
				changePalette(0);
				InitGame();
			}else{
				if(Paused){
					changePalette(0);
				}
				UpdateGame();
				RenderGame(); 
				
			}
				
			
			if(Music){
				if(Map == 0){
		
					cpc_WyzSetPlayerOff();
					cpc_WyzInitPlayer(SOUND_TABLE_1, RULE_TABLE_1, EFFECT_TABLE, SONG_TABLE_1);
					cpc_WyzLoadSong(0);
					cpc_WyzSetPlayerOn();
					Music = 0;
				
				} else if(Map > 0 && Map < 5){
					
					cpc_WyzSetPlayerOff();
					cpc_WyzInitPlayer(SOUND_TABLE_2, RULE_TABLE_2, EFFECT_TABLE, SONG_TABLE_2);
					cpc_WyzLoadSong(0);
					cpc_WyzSetPlayerOn();
					Music = 0;
					
				}else{
					
					cpc_WyzSetPlayerOff();
					cpc_WyzInitPlayer(SOUND_TABLE_3, RULE_TABLE_3, EFFECT_TABLE, SONG_TABLE_3);
					cpc_WyzLoadSong(0);
					cpc_WyzSetPlayerOn();
					Music = 0;
				
				}
				
			}
			

			
		} 
	
		if(AuxTicks > 200 && !posible){
			posible = 1;
			AuxTicks = 0;
		}else{
			AuxTicks++;
		}
		Ticks++;
		TrapTicks++;
		
		if (Exit) break;
	}

}